"""
registrar.py
=============

Contains all functions relevant for registration

"""
import logging
from typing import Optional
import json

from pystac import Item

from .source import get_source, get_source_from_path
from .backend import get_backends, get_path_backends
from .exceptions import RegistrationError
from .utils import import_by_path

logger = logging.getLogger(__name__)


def register_item(config: dict, value: str, replace: bool = False):
    """Handle registration of a stac item

    Args:
        config (dict): configuration of the registrar
        value (str): encoded STAC item to register
        replace (bool, optional): replace the file in the endpoint or not.
                                  Defaults to False.

    Raises:
        RegistrationError: Raised if item is already registered and replace is
                           not turned on
    """
    item = Item.from_dict(json.loads(value))
    source = get_source(config, item)
    logger.info(f"Handling '{item!r}'.")

    for pre_handler in get_pre_handlers(config):
        pre_handler(config, item)

    try:
        for backend in get_backends(config):
            if backend.item_exists(source, item):
                if replace:
                    logger.info(f"Replacing '{item!r}'.")
                    backend.register_item(source, item, replace=True)
                else:
                    raise RegistrationError(f"Object {item!r} is already registered")
            else:
                logger.info(f"Registering '{item!r}'.")
                backend.register_item(source, item, replace=False)
    except Exception as e:
        for error_handler in get_error_handlers(config):
            error_handler(config, item, e)
        raise
    else:
        for post_handler in get_post_handlers(config):
            post_handler(config, item)

        logger.info(
            f"Successfully {'replaced' if replace else 'registered'} " f"'{item!r}'"
        )


def register_path(config: dict, path: str, replace: bool = False):
    """Handle registration of a path

    Args:
        config (dict): configuration of the registrar
        path (str): path to register
        replace (bool, optional): replace the file in the endpoint or not.
                                  Defaults to False.

    Raises:
        RegistrationError: Raised if file is already registered and replace is
                           not turned on
    """

    source = get_source_from_path(config, path)
    logger.info(f"Handling '{path!r}'.")

    for pre_handler in get_path_pre_handlers(config):
        pre_handler(config, path)

    try:
        for backend in get_path_backends(config):
            if backend.path_exists(source, path):
                if replace:
                    logger.info(f"Replacing '{path!r}'.")
                    backend.register_path(source, path, replace=True)
                else:
                    raise RegistrationError(f"Path {path!r} is already registered")
            else:
                logger.info(f"Registering '{path!r}'.")
                backend.register_path(source, path, replace=False)
    except Exception as e:
        for error_handler in get_path_error_handlers(config):
            error_handler(config, path, e)
        raise
    else:
        for post_handler in get_path_post_handlers(config):
            post_handler(config, path)

        logger.info(
            f"Successfully {'replaced' if replace else 'registered'} " f"'{path!r}'"
        )


def deregister_item(config: dict, value: str, _: bool = False):
    item = Item.from_dict(json.loads(value))
    logger.info(f"Handling deregistration for '{item!r}'.")
    return _deregister_identifier(config, item.id, item)


def deregister_identifier(config, identifier: str, _: bool = False):
    logger.info(f"Handling deregistration of identifier '{identifier}'.")
    return _deregister_identifier(config, identifier)


def _deregister_identifier(config, identifier: str, item: Optional[Item] = None):

    for pre_handler in get_deregister_pre_handlers(config):
        pre_handler(config, identifier, item)

    try:
        for backend in get_backends(config):
            backend.deregister_identifier(identifier)
    except Exception as e:
        for error_handler in get_deregister_error_handlers(config):
            error_handler(config, identifier, item, e)
        raise
    else:
        for post_handler in get_deregister_post_handlers(config):
            post_handler(config, identifier, item)

        logger.info(f"Successfully deregistered '{identifier}'")


def deregister_path(config, path: str, _: bool = False):
    source = get_source_from_path(config, path)
    logger.info(f"Deregistering '{path!r}'.")

    for pre_handler in get_deregister_path_pre_handlers(config):
        pre_handler(config, path)

    try:
        for backend in get_path_backends(config):
            backend.deregister_path(source, path)
    except Exception as e:
        for error_handler in get_deregister_path_error_handlers(config):
            error_handler(config, path, e)
        raise
    else:
        for post_handler in get_deregister_path_post_handlers(config):
            post_handler(config, path)

        logger.info(f"Successfully deregistered '{path}'")


def _get_handlers(config, name):
    handlers = []
    for handler_def in config.get(name, []):
        handler_cls = import_by_path(handler_def["path"])
        handlers.append(
            handler_cls(
                *handler_def.get("args", []),
                **handler_def.get("kwargs", []),
            )
        )

    return handlers


def get_pre_handlers(config):
    return _get_handlers(config, "pre_handlers")


def get_post_handlers(config):
    return _get_handlers(config, "post_handlers")


def get_error_handlers(config):
    return _get_handlers(config, "error_handlers")


def get_path_pre_handlers(config):
    return _get_handlers(config, "path_pre_handlers")


def get_path_post_handlers(config):
    return _get_handlers(config, "path_post_handlers")


def get_path_error_handlers(config):
    return _get_handlers(config, "path_error_handlers")


def get_deregister_pre_handlers(config):
    return _get_handlers(config, "deregister_pre_handlers")


def get_deregister_post_handlers(config):
    return _get_handlers(config, "deregister_post_handlers")


def get_deregister_error_handlers(config):
    return _get_handlers(config, "deregister_error_handlers")


def get_deregister_path_pre_handlers(config):
    return _get_handlers(config, "deregister_path_pre_handlers")


def get_deregister_path_post_handlers(config):
    return _get_handlers(config, "deregister_path_post_handlers")


def get_deregister_path_error_handlers(config):
    return _get_handlers(config, "deregister_path_error_handlers")
